unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Registry, ExtCtrls, UrlLabel;

type
  TMSTCPType = (mtRWIN, mtTTL, mtMTUDiscovery, mtMTUBlackHole);
  
  TMainForm = class(TForm)
    Button1: TBUTTON;
    Button2: TBUTTON;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    gbxMTU: TGroupBox;
    TabSheet2: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    rbtMTUDefault: TRadioButton;
    rbtRecommand: TRadioButton;
    rbtMTUCustom: TRadioButton;
    txtMTUValue: TEdit;
    Label6: TLabel;
    Label10: TLabel;
    Button3: TBUTTON;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    lblRWINValue: TLabel;
    gbxRWIN: TGroupBox;
    rbtDefault: TRadioButton;
    rbtMultiple: TRadioButton;
    txtRWINValue: TEdit;
    rbtCustom: TRadioButton;
    gbxRecommand: TGroupBox;
    ListBox1: TListBox;
    gbxNDICache: TRadioGroup;
    gbxTTL: TRadioGroup;
    gbxMTUDetect: TGroupBox;
    cbxMTUAuto: TCheckBox;
    cbxMTUHole: TCheckBox;
    cbxDefaultMTU: TComboBox;
    tbarMultiple: TTrackBar;
    lblMultiple: TLabel;
    lbxDefaultMTUValue: TListBox;
    lblMTUValue: TLabel;
    UrlLabel1: TUrlLabel;
    UrlLabel2: TUrlLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure rbtMTUDefaultClick(Sender: TObject);
    procedure txtMTUValueExit(Sender: TObject);
    procedure txtRWINValueExit(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure cbxDefaultMTUChange(Sender: TObject);
    procedure tbarMultipleChange(Sender: TObject);
    procedure rbtCustomClick(Sender: TObject);
  private
    { Private declarations }
    FMTU, RWIN, oldMTU, oldRWIN, RWINTimes: Integer;
    oldTTL, oldNDICache, oldMTUAuto, oldMTUBlackHole: Integer;

    FMTURegistryKey: string;
    
    function GetMSTCPValue(Kind: TMSTCPType): Integer;
    procedure SetMSTCPValue(Kind: TMSTCPType; const Value: Integer);
    function GetMTU: Integer;
    procedure SetMTU(const Value: Integer);
    function GetNDICache: Integer;
    procedure SetNDICache(const Value: Integer);

    procedure ShowStatus;
    procedure ShowOldValue;
    procedure UpdateLabel;
  public
    { Public declarations }
  end;

const
  MSTCPKeys: array[TMSTCPType] of string =
    ('DefaultRcvWindow', 'DefaultTTL', 'PMTUDiscovery', 'PMTUBlackHoleDetect');

var
  MainForm: TMainForm;

implementation

uses util;

{$R *.DFM}

procedure TMainForm.SetMSTCPValue(Kind: TMSTCPType; const Value: Integer);
begin
  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey('System\CurrentControlSet\Services\VxD\MSTCP', True) = False then Exit;
      WriteString(MSTCPKeys[Kind], IntToStr(Value));
    finally
      CloseKey;
      Free;
    end;
  end;
end;

function TMainForm.GetMSTCPValue(Kind: TMSTCPType): Integer;
const
  bEverErrBox: Boolean = False;
var
  S: string;
begin
  Result := -1;
  
  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey('System\CurrentControlSet\Services\VxD\MSTCP', False) = False then
      begin
        // P˪~TX{@NF..
        if not bEverErrBox then ErrBox('䤣X \HKLM\System\CurrentControlSet\Services\VxD\MSTCP');
        bEverErrBox := True;
        
        case Kind of
          mtRWIN:
          begin
            EnableControl(gbxRWIN, False);
            EnableControl(lblRWINValue, False);
            EnableControl(lblMTUValue, False);
          end;
          mtTTL: EnableControl(gbxTTL, False);
          mtMTUDiscovery, mtMTUBlackHole: EnableControl(gbxMTUDetect, False);
        end;
        Exit;
      end;
      
      try
        S := ReadString(MSTCPKeys[Kind]);
      except
        // YϬO Integer A]nŪJ
        try
          Result := ReadInteger(MSTCPKeys[Kind]);
          Exit; // ŪFKYX, U StrToIntDef ʧ@
        except
          // YŪ]nҥ~o..
        end;
      end;
      Result := StrToIntDef(S, - 1); // -1 S]w
      
    finally
      CloseKey;
      Free;
    end;
  end;
end;

function TMainForm.GetNDICache: Integer;
var
  S: string;
begin
  Result := -1;

  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey('System\CurrentControlSet\Services\VxD\NWLink\Ndi\params\cachesize', False) = False then
      begin
        ErrBox('䤣X \HKLM\System\CurrentControlSet\Services\VxD\NWLink\Ndi\params\cachesize');
        EnableControl(gbxNDICache, False);
        Exit;
      end;

      try
        S := ReadString(''); // default value
      except
      end;
      Result := StrToIntDef(S, - 1); // -1 S]w

    finally
      CloseKey;
      Free;
    end;
  end;
end;

procedure TMainForm.SetNDICache(const Value: Integer);
begin
  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey('System\CurrentControlSet\Services\VxD\NWLink\Ndi\params\cachesize', True) = False then Exit;
      WriteString('', IntToStr(Value));
    finally
      CloseKey;
      Free;
    end;
  end;
end;

function TMainForm.GetMTU: Integer;
const
  MAX_SET_NO = 100;
var
  SetNo, I, J: Integer;    
  S          : string;     
  SL         : TStringList;
begin
  Result := -1;
  FMTURegistryKey := '';
  EnableControl(gbxMTU, False); // disable it, Ū\~ enable
  
  with TRegistry.Create do
  begin
    SL := TStringList.Create();
    try
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey('Enum\Root\Net', False) = False then
      begin
        ErrBox('䤣X \HKLM\Enum\Root\Net');
        Exit;
      end;
      
      SetNo := 0;
      repeat
        S := Format('%4.4d',[SetNo]);
        Inc(SetNo);
      until KeyExists(S) or (SetNo > MAX_SET_NO);
      
      if SetNo > MAX_SET_NO then
      begin
        ErrBox('䤣X \HKLM\Enum\Root\Net\????');
        Exit;
      end;
      
      if OpenKey(S + '\Bindings', False) = False then
      begin
        ErrBox('䤣X \HKLM\Enum\Root\Net\' + S + '\Bindings');
        Exit;
      end;
      
      GetValueNames(SL);
      J := SL.Count - 1;
      I := -1;
      repeat
        if (Pos('MSTCP', SL.Strings[J]) > 0) then I := J;
        dec(J);
      until J = -1;
      
      if I = -1 then
      begin
        ErrBox('䤣X \HKLM\Enum\Root\Net\' + S + '\Bindings\MSTCP\????');
        Exit;
      end;
      
      if OpenKey('\Enum\Network\' + SL[I], False) = False then
      begin
        ErrBox('䤣X \HKLM\Enum\Network\' + SL[I]);
        Exit;
      end;
      
      try
        S := ReadString('Driver');
      except
        ErrBox('Ū \HKLM\Enum\Network\' + SL[I] + '\Driver');
        Exit;
      end;
      
      if OpenKey('\System\CurrentControlSet\Services\Class\' + S, False) = False then
      begin
        ErrBox('䤣X \HKLM\System\CurrentControlSet\Services\Class\' + S);
        Exit;
      end;
      
      // N MTU XO_, xsɴNέ@MF !!
      FMTURegistryKey := '\System\CurrentControlSet\Services\Class\' + S;
      
      EnableControl(gbxMTU, True); // \X...
      
      try
        S := ReadString('MaxMTU');
      except
        // ...
        try
          Result := ReadInteger('MaxMTU');
          Exit; // ŪFKYX, U StrToIntDef ʧ@
        except
          // YŪ]nҥ~o..
        end;
      end;
      Result := StrToIntDef(S, - 1); // -1 S]w
      
    finally
      CloseKey;
      SL.Free();
      Free;
    end;
  end;
  
end;

procedure TMainForm.SetMTU(const Value: Integer);
begin
  with TRegistry.Create do
  begin
    try
      if FMTURegistryKey = '' then Exit;
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey(FMTURegistryKey, True) = False then
      begin
        ErrBox('䤣X ' + FMTURegistryKey);
        Exit;
      end;
      WriteString('MaxMTU', IntToStr(Value));
    finally
      CloseKey;
      Free;
    end;
  end;

end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  INFO: TOSVersionInfo;
begin
  PageControl1.ActivePage := TabSheet1;
  INFO.dwOSVersionInfoSize := sizeof(INFO);
  GetVersionEx(INFO);
  if INFO.dwPlatformId <> VER_PLATFORM_WIN32_WINDOWS then
  begin
    EnableControl(gbxRWIN, False);

    EnableControl(lblRWINValue, False);
    EnableControl(lblMTUValue, False);
    EnableControl(gbxMTU, False);

    EnableControl(gbxTTL, False);
    EnableControl(gbxNDICache, False);
    EnableControl(gbxMTUDetect, False);


    ErrBox('ܩp, {uAΩ Windows 95 !!');
    WinHelp(Handle, PChar('xmtucfg.hlp'), HELP_CONTEXT, 6);
  end else
  begin
    cbxDefaultMTU.ItemIndex := 5;
    
    RWINTimes := -1;
    FMTU := GetMTU;
    oldMTU := FMTU;
    RWIN := GetMSTCPValue(mtRWIN);
    oldRWIN := RWIN;
    
    oldTTL := GetMSTCPValue(mtTTL);
    oldNDICache := GetNDICache;
    
    oldMTUAuto := GetMSTCPValue(mtMTUDiscovery);
    oldMTUBlackHole := GetMSTCPValue(mtMTUBlackHole);
    
    cbxMTUAuto.checked := oldMTUAuto = 1;
    cbxMTUHole.checked := oldMTUBlackHole = 1;
    
    ShowOldValue;
    ShowStatus;
  end;
  
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Value   : Integer;
  bChanged: Boolean;
begin
  if ModalResult = mrOk then
  begin
    if FMTU <> -1 then SetMTU(FMTU);
    if RWIN <> -1 then SetMSTCPValue(mtRWIN, RWIN);
    
    bChanged := (FMTU <> oldMTU) or (RWIN <> oldRWIN);
    
    if gbxTTL.ENABLED and (gbxTTL.ItemIndex <> -1) then
    begin
      Value := (gbxTTL.ItemIndex + 1) * 32;
      bChanged := bChanged or (oldTTL <> Value);
      SetMSTCPValue(mtTTL, Value);
    end;
    
    if gbxNDICache.ENABLED and (gbxNDICache.ItemIndex <> -1) then
    begin
      Value := gbxNDICache.ItemIndex * 16;
      if Value = 48 then Value := 64;
      bChanged := bChanged or (oldNDICache <> Value);
      SetNDICache(Value);
    end;
    
    if gbxMTUDetect.ENABLED then
    begin
      if cbxMTUAuto.checked then Value := 1 else Value := 0;
      bChanged := bChanged or (oldMTUAuto <> Value);
      SetMSTCPValue(mtMTUDiscovery, Value);
      
      if cbxMTUHole.checked then Value := 1 else Value := 0;
      bChanged := bChanged or (oldMTUBlackHole <> Value);
      SetMSTCPValue(mtMTUBlackHole, Value);
      
    end;
    
    if bChanged then MsgBox('ЭsҰʹqAs]w~|ͮ');
  end;
end;

procedure TMainForm.ShowStatus;
var
  I: Integer;
begin
  if (RWIN <> -1) and (FMTU <> -1) then
  begin
    txtRWINValue.Text := IntToStr(RWIN);
    if RWIN = 8192 then
    begin
      RWINTimes := 0;
      rbtDefault.checked := True;
    end
    else
    begin
      RWINTimes := RWIN div (FMTU - 40);
      if RWIN mod (FMTU - 40) <> 0 then Inc(RWINTimes);
      
      if (RWIN mod (FMTU - 40) = 0) and (RWINTimes >= 2) and (RWINTimes <= 10) then
      begin
        tbarMultiple.ENABLED := True;
        tbarMultiple.Position := RWINTimes;
        tbarMultipleChange(tbarMultiple);
        rbtMultiple.checked := True;
      end else
      begin
        txtRWINValue.ENABLED := True;
        RWINTimes := 1;
        rbtCustom.checked := True;
      end;
    end;
  end;
  
  if FMTU <> -1 then
  begin
    txtMTUValue.Text := IntToStr(FMTU);
    if FMTU = 1500 then rbtMTUDefault.checked := True
    else
    begin
      I := lbxDefaultMTUValue.Items.IndexOf(IntToStr(FMTU));
      if I <> -1 then
      begin
        cbxDefaultMTU.ItemIndex := I;
        rbtRecommand.checked := True;
      end else
      begin
        txtMTUValue.ENABLED := True;
        rbtMTUCustom.checked := True;
      end;
    end;
  end;
  
  case oldTTL of
     - 1, 32: gbxTTL.ItemIndex := 0; // -1 = default
    64: gbxTTL.ItemIndex := 1;
  else
    gbxTTL.ItemIndex := -1;
  end;
  
  case oldNDICache of
     - 1, 0: gbxNDICache.ItemIndex := 0; // -1 = default
    16: gbxNDICache.ItemIndex := 1;
    32: gbxNDICache.ItemIndex := 2;
    64: gbxNDICache.ItemIndex := 3;
  else
    gbxNDICache.ItemIndex := -1;
  end;
  
  UpdateLabel;
end;

procedure TMainForm.ShowOldValue;
begin
  if oldMTU <> -1 then gbxMTU.CAPTION := 'MTU (]w=' + IntToStr(oldMTU) + ')'
  else gbxMTU.CAPTION := 'MTU (]w)';
  
  if oldRWIN <> -1 then gbxRWIN.CAPTION := 'RWIN (]w=' + IntToStr(oldRWIN) + ')'
  else gbxRWIN.CAPTION := 'RWIN (]w)';

  if oldTTL <> -1 then gbxTTL.CAPTION := 'TTL (]w=' + IntToStr(oldTTL) + ')'
  else gbxTTL.CAPTION := 'TTL (]w)';
  
  if oldNDICache <> -1 then gbxNDICache.CAPTION := 'NDI Cache Size (]w=' + IntToStr(oldNDICache) + ')'
  else gbxNDICache.CAPTION := 'NDI Cache Size (]w)';
end;

procedure TMainForm.UpdateLabel;
begin
  if FMTU = -1 then
    lblMTUValue.CAPTION := 'MTU ثeS]w'
  else
    lblMTUValue.CAPTION := 'MTU ثe]w: ' + IntToStr(FMTU);
  
  if gbxRWIN.ENABLED then
  begin
    case RWINTimes of
      0: RWIN := 8192;
      1: try
          RWIN := StrToInt(txtRWINValue.Text);
        except
        end
    else
      if RWINTimes <> -1 then RWIN := (FMTU - 40) * RWINTimes;
    end;
    
    if RWIN = -1 then
      lblRWINValue.CAPTION := 'RWIN ثeS]w'
    else
      lblRWINValue.CAPTION := 'RWIN ثe]w: ' + IntToStr(RWIN);
  end;
end;

procedure TMainForm.rbtMTUDefaultClick(Sender: TObject);
begin
  txtMTUValue.ENABLED := Sender = rbtMTUCustom;
  cbxDefaultMTU.ENABLED := Sender = rbtRecommand;
  if (Sender as TComponent).Tag <> 0 then FMTU := (Sender as TComponent).Tag;
  UpdateLabel;
end;

procedure TMainForm.txtMTUValueExit(Sender: TObject);
begin
  try
    FMTU := StrToInt(txtMTUValue.Text);
  except
    ErrBox('жJƼƭ !!');
    txtMTUValue.SelectAll;
    txtMTUValue.SetFocus;
  end;
  
  if FMTU <= 512 then
  begin
    ErrBox('MTU Ȧܤ֥ 513 !!');
    txtMTUValue.Text := '513';
    FMTU := 513;
    txtMTUValue.SelectAll;
    txtMTUValue.SetFocus;
  end;
  UpdateLabel;
end;

procedure TMainForm.txtRWINValueExit(Sender: TObject);
begin
  try
    RWIN := StrToInt(txtRWINValue.Text);
  except
    ErrBox('жJƼƭ !!');
    txtRWINValue.SelectAll;
    txtRWINValue.SetFocus;
  end;
  UpdateLabel;
end;


procedure TMainForm.Button3Click(Sender: TObject);
begin
  WinHelp(Handle, PChar('xmtucfg.hlp'), HELP_CONTENTS, 0);
end;

procedure TMainForm.cbxDefaultMTUChange(Sender: TObject);
begin
  if cbxDefaultMTU.ItemIndex = -1 then cbxDefaultMTU.ItemIndex := cbxDefaultMTU.Items.Count - 1;
  rbtRecommand.CAPTION := lbxDefaultMTUValue.Items[cbxDefaultMTU.ItemIndex];
  FMTU := StrToInt(rbtRecommand.CAPTION);
  rbtRecommand.Tag := FMTU;
  UpdateLabel;
end;


procedure TMainForm.tbarMultipleChange(Sender: TObject);
begin
  lblMultiple.CAPTION := ': ' + IntToStr(tbarMultiple.Position);
  RWINTimes := tbarMultiple.Position;
  rbtMultiple.Tag := RWINTimes;
  UpdateLabel;
end;

procedure TMainForm.rbtCustomClick(Sender: TObject);
begin
  txtRWINValue.ENABLED := Sender = rbtCustom;
  tbarMultiple.ENABLED := Sender = rbtMultiple;
  RWINTimes := (Sender as TComponent).Tag;
  UpdateLabel;
end;

end.

