unit mapunit;

interface

uses Windows, SysUtils, Graphics, Classes, tiles, util;

type
  TMapArray = array[0..TILE_NUM_Y - 1, 0..TILE_NUM_X - 1] of Byte;
  
  tmap = class
  private
    FTerrMap, FItemMap: TMapArray; // aΤΪ~a
    
    FLevelNo: Integer; // ثeJds
    FInvBiTMap: TBitmap; // Ω󪫫~zK
    
    FRole_X, FRole_Y: Integer; // ⪺_lm
    
    procedure SetLevelNo(const Value: Integer);
    
    function GetCanPass(X, Y: Integer): Boolean;
    function GetIsTarget(X, Y: Integer): Boolean;
    
    function GetCanMove(X, Y: Integer): Boolean;
    function GetIsSource(X, Y: Integer): Boolean;
    
    function GetTerrAttr(X, Y: Integer): TTerrAttr;
    procedure SetTerrAttr(X, Y: Integer; const Value: TTerrAttr);
    function GetItemAttr(X, Y: Integer): TItemAttr;
    procedure SetItemAttr(X, Y: Integer; const Value: TItemAttr);
    
    function GetTerrMap(X, Y: Integer): Byte;
    function GetItemMap(X, Y: Integer): Byte;
    procedure SetTerrMap(X, Y: Integer; const Value: Byte);
    procedure SetItemMap(X, Y: Integer; const Value: Byte);
    procedure SetRole_X(const Value: Integer);
    procedure SetRole_Y(const Value: Integer);
  protected
  public
    constructor Create;
    destructor Destroy; override;
    
    function GetFileName: string; // ھds, Ǧ^ɦW
    
    procedure LoadFromFile;
    procedure SaveToFile;
    
    procedure DrawTerrMap(Canvas: TCanvas); // Naεeb Canvas W
    procedure DrawItemMap(Canvas: TCanvas); // N~eb Canvas W
    
    // ]ia, Υu]aΩΪ~
    procedure ResetMap;
    procedure ResetTerrs;
    procedure ResetItems;
    
    property LevelNo: Integer read FLevelNo write SetLevelNo;
    
    // oY檺aΤΪ~Ϥs
    property TerrMap[X, Y: Integer]: Byte read GetTerrMap write SetTerrMap;
    property ItemMap[X, Y: Integer]: Byte read GetItemMap write SetItemMap;
    
    // olm
    property Role_X: Integer read FRole_X write SetRole_X;
    property Role_Y: Integer read FRole_Y write SetRole_Y;
    
    // oY檺aݩ
    property TerrAttr[X, Y: Integer]: TTerrAttr read GetTerrAttr write SetTerrAttr;
    
    property CanPass[X, Y: Integer]: Boolean read GetCanPass;
    property IsTarget[X, Y: Integer]: Boolean read GetIsTarget;
    
    // oY檺~ݩ
    property ItemAttr[X, Y: Integer]: TItemAttr read GetItemAttr write SetItemAttr;
    
    property CanMove[X, Y: Integer]: Boolean read GetCanMove;
    property IsSource[X, Y: Integer]: Boolean read GetIsSource;
  end;
  
var
  Map: tmap;
  
implementation

uses xFiles;

constructor tmap.Create;
begin
  inherited Create;
  
  FLevelNo := 0;
  
  FInvBiTMap := TBitmap.Create;
  FInvBiTMap.Width := TILE_WIDTH;
  FInvBiTMap.Height := TILE_Height;
  FInvBiTMap.Transparent := True;
  FInvBiTMap.TransparentColor := RGB(128, 0, 0);
end;

destructor tmap.Destroy;
begin
  FInvBiTMap.Free;
  
  inherited Destroy;
end;

function tmap.GetFileName: string;
begin
  Result := Format('%s%s%d%s',[AppDir, FN_MAP_PREFIX, FLevelNo, FN_MAP_EXT]);
end;

procedure tmap.LoadFromFile;
var
  fs: TFileStream;
begin
  fs := TFileStream.Create(GetFileName, fmOpenRead);
  with fs do
    try
      CheckSignature(fs, SIG_MYFILE);
      CheckSignature(fs, ClassName);
      
      read (FRole_X, sizeof(Integer));
      read (FRole_Y, sizeof(Integer));
      
      read (FTerrMap, sizeof(TMapArray));
      read (FItemMap, sizeof(TMapArray));
    finally
      Free;
    end;
end;

procedure tmap.SaveToFile;
var
  fs: TFileStream;
  S : string;     
begin
  fs := TFileStream.Create(GetFileName, fmCreate or fmOpenWrite);
  with fs do
    try
      write (SIG_MYFILE, Length(SIG_MYFILE));
      S := ClassName;
      write (S[1], Length(S));
      
      write (FRole_X, sizeof(Integer));
      write (FRole_Y, sizeof(Integer));
      
      write (FTerrMap, sizeof(TMapArray));
      write (FItemMap, sizeof(TMapArray));
    finally
      Free;
    end;
end;

procedure tmap.SetLevelNo(const Value: Integer);
begin
  FLevelNo := Value;
  LoadFromFile;
end;

function tmap.GetCanPass(X, Y: Integer): Boolean;
begin
  Result := taCanPass in Terrs.Attrs[FTerrMap[Y, X]];
end;

function tmap.GetIsTarget(X, Y: Integer): Boolean;
begin
  Result := taTarget in Terrs.Attrs[FTerrMap[Y, X]];
end;

function tmap.GetTerrAttr(X, Y: Integer): TTerrAttr;
begin
  Result := Terrs.Attrs[FTerrMap[Y, X]];
end;

procedure tmap.SetTerrAttr(X, Y: Integer; const Value: TTerrAttr);
begin
  Terrs.Attrs[FTerrMap[Y, X]] := Value;
end;

function tmap.GetItemAttr(X, Y: Integer): TItemAttr;
begin
  Result := Items.Attrs[FItemMap[Y, X]];
end;

procedure tmap.SetItemAttr(X, Y: Integer; const Value: TItemAttr);
begin
  Items.Attrs[FItemMap[Y, X]];
end;

function tmap.GetCanMove(X, Y: Integer): Boolean;
begin
  Result := iaCanMove in Items.Attrs[FItemMap[Y, X]];
end;

function tmap.GetIsSource(X, Y: Integer): Boolean;
begin
  Result := iaSource in Items.Attrs[FItemMap[Y, X]];
end;

procedure tmap.DrawTerrMap(Canvas: TCanvas);
var
  X, Y: Integer;
begin
  for Y := 0 to TILE_NUM_Y - 1 do
    for X := 0 to TILE_NUM_X - 1 do
      BitBlt(Canvas.Handle, X * TILE_WIDTH, Y * TILE_Height, TILE_WIDTH, TILE_Height,
      Terrs.Bitmap.Canvas.Handle, Terrs.TilePos_Left[FTerrMap[Y, X]], Terrs.TilePos_Top[FTerrMap[Y, X]], SRCCOPY);
end;

procedure tmap.DrawItemMap(Canvas: TCanvas);
var
  X, Y     : Integer;
  Left, Top: Integer;
begin
  // Τ@ FInvBitmap  Items.Bitmap 榡
  FInvBiTMap.PixelFormat := Items.Bitmap.PixelFormat;
  
  for Y := 0 to TILE_NUM_Y - 1 do
    for X := 0 to TILE_NUM_X - 1 do
      if (FItemMap[Y, X] <> 0) then
      begin
        Left := Items.TilePos_Left[FItemMap[Y, X]];
        Top := Items.TilePos_Top[FItemMap[Y, X]];
        
        FInvBiTMap.Canvas.CopyRect(Rect(0, 0, TILE_WIDTH, TILE_Height), Items.Bitmap.Canvas,
          Rect(Left, Top, Left + TILE_WIDTH, Top + TILE_Height));
        
        // ѨM VCL b Windows 98 UzK bug  workaround
        if IsWindows98 then
          TransparentBlt(Canvas.Handle, FInvBiTMap.Handle, X * TILE_WIDTH, Y * TILE_Height, RGB(128, 0, 0))
        else
          Canvas.Draw(X * TILE_WIDTH, Y * TILE_Height, FInvBiTMap);
      end;
end;

procedure tmap.ResetMap;
begin
  ResetTerrs;
  ResetItems;
  FRole_X := 0;
  FRole_Y := 0;
end;

procedure tmap.ResetTerrs;
begin
  FillChar(FTerrMap, sizeof(FTerrMap), 0);
end;

procedure tmap.ResetItems;
begin
  FillChar(FItemMap, sizeof(FTerrMap), 0);
end;


function tmap.GetItemMap(X, Y: Integer): Byte;
begin
  Result := FItemMap[Y, X];
end;

function tmap.GetTerrMap(X, Y: Integer): Byte;
begin
  Result := FTerrMap[Y, X];
end;

procedure tmap.SetTerrMap(X, Y: Integer; const Value: Byte);
begin
  FTerrMap[Y, X] := Value;
end;

procedure tmap.SetItemMap(X, Y: Integer; const Value: Byte);
begin
  FItemMap[Y, X] := Value;
end;

procedure tmap.SetRole_X(const Value: Integer);
begin
  FRole_X := Value;
end;

procedure tmap.SetRole_Y(const Value: Integer);
begin
  FRole_Y := Value;
end;

end.

