unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls, Spin, Grids, ExtDlgs, Math, Tiles;

type
  TMainForm = class(TForm)
    mnuMain: TMainMenu;
    F1: TMenuItem;
    H1: TMenuItem;
    mnuAbout: TMenuItem;
    mnuExit: TMenuItem;
    N1: TMenuItem;
    mnuOpen: TMenuItem;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    grdPreview: TDrawGrid;
    GroupBox1: TGroupBox;
    Label4: TLabel;
    lblTileNo: TLabel;
    nbkTileAttr: TNotebook;
    mnuSave: TMenuItem;
    N2: TMenuItem;
    mnuImport: TMenuItem;
    mnuNew: TMenuItem;
    lblTileKind: TLabel;
    dlgOpen: TOpenDialog;
    dlgSave: TSaveDialog;
    mnuSaveAs: TMenuItem;
    N3: TMenuItem;
    cbxCanPass: TCheckBox;
    cbxTarget: TCheckBox;
    cbxSource: TCheckBox;
    cbxCanMove: TCheckBox;
    lblTileNum: TLabel;
    dlgOpenPicture: TOpenPictureDialog;
    procedure mnuExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mnuNewClick(Sender: TObject);
    procedure mnuOpenClick(Sender: TObject);
    procedure mnuSaveClick(Sender: TObject);
    procedure grdPreviewDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure mnuImportClick(Sender: TObject);
    procedure grdPreviewSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure mnuAboutClick(Sender: TObject);
  private
    FTiles: TTiles;
    FFileName: string;
    
    procedure UpdateControlStatus;
    procedure ChangeTileClass(TileKind: TTilesClass);
  public
    { Public declarations }
  end;
  
var
  MainForm: TMainForm;
  
implementation

uses util, kinddlg, about;

{$R *.DFM}

procedure TMainForm.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  ChangeTileClass(TTerrTiles); // w]aιϮw
  grdPreview.ColCount := FTiles.TileNumPerRow;

  UpdateControlStatus;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ;
end;

procedure TMainForm.ChangeTileClass(TileKind: TTilesClass);
begin
  if Assigned(FTiles) then FTiles.Free;

  FTiles := TileKind.Create; //  TileKind wOӫإ FTiles 
end;

procedure TMainForm.mnuNewClick(Sender: TObject);
begin
  with TTileKindDlg.Create(self) do // إ߸߰ݹܲ
    try
      if ShowModal = mrOk then // qXܲ, ϥΪ̫U"Tw"|Ǧ^ mrOK
      begin
        if rgptilekind.ItemIndex = 0 then
          ChangeTileClass(TTerrTiles)    // aιϮw
        else
          ChangeTileClass(TItemTiles);   // ~Ϯw

        UpdateControlStatus;
      end;
    finally
      Free; // Lצp, R߰ݹܲ
    end;
end;

procedure TMainForm.mnuOpenClick(Sender: TObject);
begin
  if dlgOpen.Execute then
    try
      try
        ChangeTileClass(TTerrTiles); // ոլݬO_aιϮw
        // Ū, N|ͨҥ~, ڭ̪ҥ~ɮBz(except ᨺ@q)
        FTiles.LoadFromFile(dlgOpen.Filename);
        FFileName := dlgOpen.Filename; // ŪaιϮw\
      except
        // pGOaιϮw, ˬdYҮɷ|ͨҥ~, ҥHoӰ
        ChangeTileClass(TItemTiles); // AոլݬO_~Ϯw, _hN򳣤O
        // , ]|ͨҥ~, ڭ̴NBzF
        FTiles.LoadFromFile(dlgOpen.Filename);
        FFileName := dlgOpen.Filename;
      end;
    finally
      UpdateControlStatus;
    end;
end;

procedure TMainForm.mnuSaveClick(Sender: TObject);
begin
  // YO"tss" ٥wɦW, NݨϥΪɦW
  if ((Sender as TComponent).Tag = 1) or (FFileName = '') then
  begin
    dlgSave.Filter := dlgOpen.Filter;
    if not dlgSave.Execute then Exit; // ߰ݨϥΪɦW, YN}
    FFileName := dlgSave.Filename; // NɦWO_
  end;

  FTiles.SaveToFile(FFileName); // xsϮw
  UpdateControlStatus; // sD
end;

procedure TMainForm.UpdateControlStatus;
const
  TileKindStr: array[Boolean] of string = ('Terrain', 'Items');
var
  sFileName: string;
begin
  // update caption
  sFileName := FFileName;
  if sFileName = '' then sFileName := 'Untitled';
  Caption := Format('%s - %s',[Application.Title, sFileName]);

  if FTiles is TTerrTiles then nbkTileAttr.PageIndex := 0
  else nbkTileAttr.PageIndex := 1;
  lblTileNum.Caption := IntToStr(FTiles.TileNum);
  lblTileKind.Caption := TileKindStr[not (FTiles is TTerrTiles)];

  mnuSave.Enabled := FTiles.TileNum <> 0;
  mnuSaveAs.Enabled := FTiles.TileNum <> 0;

  grdPreview.RowCount := Max(10, FTiles.TileRowCount);
  grdPreview.Invalidate;
end;

procedure TMainForm.grdPreviewDrawCell(Sender: TObject; ACol,
    ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  No: Integer;
begin
  No := ARow * grdPreview.ColCount + ACol; // ѦΦCϤs

  with grdPreview.Canvas do
    if No < FTiles.TileNum then // NϧεeX
      BitBlt(Handle, Rect.Left, Rect.Top, TILE_WIDTH, TILE_HEIGHT,
      FTiles.Bitmap.Canvas.Handle, FTiles.TilePos_Left[No], FTiles.TilePos_Top[No], SRCCOPY)
    else
    begin
      Brush.Color := clBlack; // sjϤƥ
      FillRect(Rect);
    end;
end;

procedure TMainForm.mnuImportClick(Sender: TObject);
begin
  if dlgOpenPicture.Execute then
  begin // פJ BMP ϧ
    FTiles.ImportPicture(dlgOpenPicture.Filename);
    UpdateControlStatus;
  end;
end;

procedure TMainForm.grdPreviewSelectCell(Sender: TObject; ACol,
    ARow: Integer; var CanSelect: Boolean);
var
  OldNo, No: Integer;
begin
  with grdPreview do
  begin
    OldNo := Row * grdPreview.ColCount + Col; // p쥻ܪϤs
    if OldNo < FTiles.TileNum then // NϥΪ̳]wϤݩʼg^h
    begin
      if FTiles is TTerrTiles then // YOaιϮw...
      begin // ھ cbxCanPass  cbxTarget  checkbox ӳ]wϤݩ
        TTerrTiles(FTiles).Attrs[OldNo] := [];

        // iHVa
        if cbxCanPass.Checked then TTerrTiles(FTiles).Attrs[OldNo] :=
          TTerrTiles(FTiles).Attrs[OldNo] + [taCanpass];

        // Oتa
        if cbxTarget.Checked then TTerrTiles(FTiles).Attrs[OldNo] :=
          TTerrTiles(FTiles).Attrs[OldNo] + [taTarget];
      end else
      begin // YO~Ϯw...
        TItemTiles(FTiles).Attrs[OldNo] := [];

        // iHhʪ~
        if cbxCanMove.Checked then TItemTiles(FTiles).Attrs[OldNo] :=
          TItemTiles(FTiles).Attrs[OldNo] + [iaCanmove];

        // л\Ϊ~
        if cbxSource.Checked then TItemTiles(FTiles).Attrs[OldNo] :=
          TItemTiles(FTiles).Attrs[OldNo] + [iasource];
      end;
    end;
  end;

  No := ARow * grdPreview.ColCount + ACol; // pYNܪϤs
  if No < FTiles.TileNum then // qXثeҿ諸Ϥݩ
  begin
    lblTileNo.Caption := Format('s: %d',[No]);
    // NҿܪϤݩʥ checkbox {X
    if FTiles is TTerrTiles then
    begin
      cbxCanPass.Checked := taCanpass in TTerrTiles(FTiles).Attrs[No];
      cbxTarget.Checked := taTarget in TTerrTiles(FTiles).Attrs[No];
    end else
    begin
      cbxCanMove.Checked := iaCanmove in TItemTiles(FTiles).Attrs[No];
      cbxSource.Checked := iasource in TItemTiles(FTiles).Attrs[No];
    end;
  end else CanSelect := False; // ܤXkϮ, 
end;

procedure TMainForm.mnuAboutClick(Sender: TObject);
begin
  with TAboutBox.Create(self) do
    try
      ShowModal;
    finally
      Free;
    end;
end;

end.

