unit tiles;

interface

uses SysUtils, Classes, Graphics, util;

type
  TTerrAttr = set of (taCanPass, taTarget); // aΪݩ
  TItemAttr = set of (iaCanMove, iaSource); // ~ݩ

  // metaclass, O instance iHO TTerrTiles or TItemTiles
  TTilesClass = class of TTiles;

  TTiles = class
  private
    FBits: TBitmap; // sϮwҦϤ bitmap

    FTileNum: Integer; // Ϥƶq

    function GetTilePos_Left(index: Integer): Integer;
    function GetTilePos_Top(index: Integer): Integer;

    function GetTileNumPerRow: Integer;
    function GetTileRowCount: Integer;
  protected
    procedure SetTileNum(Value: Integer); virtual;

    // Ūγ]wݩʳOk, NOӧg
    procedure LoadAttrs(fs: TFileStream); virtual; abstract;
    procedure WriteAttrs(fs: TFileStream); virtual; abstract;
  public
    constructor Create;
    destructor Destroy; override;

    // JxsϮw
    procedure LoadFromFile(Filename: string);
    procedure SaveToFile(Filename: string);

    procedure ImportPicture(Filename: string); // פJϧ

    property TileNum: Integer read FTileNum write SetTileNum;

    property TileNumPerRow: Integer read GetTileNumPerRow; // CCϤƥ
    property TileRowCount: Integer read GetTileRowCount; // @XCϤ

    property Bitmap: TBitmap read FBits;

    // ھڹϤsNioϤbϧΤm
    property TilePos_Left[index: Integer]: Integer read GetTilePos_Left;
    property TilePos_Top[index: Integer]: Integer read GetTilePos_Top;
  end;

  TTerrTiles = class(TTiles)
  private
    FAttrs: array of TTerrAttr;
    function GetAttrs(index: Integer): TTerrAttr;
    procedure SetAttrs(index: Integer; const Value: TTerrAttr);
  protected
    procedure SetTileNum(Value: Integer); override;
    
    procedure LoadAttrs(fs: TFileStream); override;
    procedure WriteAttrs(fs: TFileStream); override;
  public
    property Attrs[index: Integer]: TTerrAttr read GetAttrs write SetAttrs;
  end;
  
  TItemTiles = class(TTiles)
  private
    FAttrs: array of TItemAttr;
    function GetAttrs(index: Integer): TItemAttr;
    procedure SetAttrs(index: Integer; const Value: TItemAttr);
  protected
    procedure SetTileNum(Value: Integer); override;
    
    procedure LoadAttrs(fs: TFileStream); override;
    procedure WriteAttrs(fs: TFileStream); override;
  public
    property Attrs[index: Integer]: TItemAttr read GetAttrs write SetAttrs;
  end;
  
var
  Terrs: TTerrTiles;
  Items: TItemTiles;
  
implementation

const
  TILE_BITS_NUM_X = 10; // ϮwCCϤƥ
  
constructor TTiles.Create;
begin
  inherited Create;
  
  FTileNum := 0;
  FBits := TBitmap.Create;
end;

destructor TTiles.Destroy;
begin
  FBits.Free;
  
  inherited Destroy;
end;

procedure TTiles.LoadFromFile(Filename: string);
var
  fs : TFileStream;
  Val: Integer;
begin
  fs := TFileStream.Create(Filename, fmOpenRead);
  try
    CheckSignature(fs, SIG_MYFILE); // ˬdY
    CheckSignature(fs, ClassName); // ˬdY

    fs.read (Val, sizeof(Integer)); // ŪJϤƥ
    if Val = 0 then
      raise Exception.Create('No tiles');
    TileNum := Val; // FĲo property  SetXXX method

    // oOk, ] TTiles ڥSwqݩ, NOMwӦpŪݩ
    LoadAttrs(fs);

    FBits.LoadFromStream(fs); // ŪJsҦϤϧ

    // ˬdϧΪeŤŦXз
    if FBits.Width < TILE_WIDTH * TILE_BITS_NUM_X then
      raise Exception.Create('Width of tile bitmap is invalid');

    if FBits.Height div TILE_HEIGHT * TILE_BITS_NUM_X < FTileNum then
      raise Exception.Create('Height of tile bitmap is invalid'); // too small tiles
  finally
    fs.Free;
  end;
end;

procedure TTiles.SaveToFile(Filename: string);
var
  fs: TFileStream;
  S : string;
begin
  fs := TFileStream.Create(Filename, fmCreate or fmOpenWrite);
  try
    fs.write (SIG_MYFILE, Length(SIG_MYFILE));
    S := ClassName;
    fs.write (S[1], Length(S));

    fs.write (FTileNum, sizeof(Integer));
    
    WriteAttrs(fs);
    
    FBits.SaveToStream(fs);
  finally
    fs.Free;
  end;
end;

procedure TTiles.ImportPicture(Filename: string);
begin
  FBits.LoadFromFile(Filename);
  TileNum := FBits.Height div TILE_HEIGHT * TILE_BITS_NUM_X;
  
  // N bitmap һݪjp
  FBits.Width := TILE_WIDTH * TILE_BITS_NUM_X;
  FBits.Height := TileNum div TILE_BITS_NUM_X * TILE_HEIGHT;
end;

procedure TTiles.SetTileNum(Value: Integer);
begin
  if Value <> FTileNum then
  begin
    FTileNum := Value;
  end;
end;

function TTiles.GetTilePos_Left(index: Integer): Integer;
begin
  Result := (index mod TILE_BITS_NUM_X) * TILE_WIDTH;
end;

function TTiles.GetTilePos_Top(index: Integer): Integer;
begin
  Result := (index div TILE_BITS_NUM_X) * TILE_HEIGHT;
end;

function TTiles.GetTileNumPerRow: Integer;
begin
  Result := TILE_BITS_NUM_X;
end;

function TTiles.GetTileRowCount: Integer;
begin
  Result := FBits.Height div TILE_HEIGHT;
end;

// implementation of TTerrTiles class

procedure TTerrTiles.SetTileNum(Value: Integer);
begin
  inherited SetTileNum(Value);
  SetLength(FAttrs, FTileNum);
end;

procedure TTerrTiles.LoadAttrs(fs: TFileStream);
begin
  fs.read (FAttrs[0], sizeof(TTerrAttr) * FTileNum);
end;

procedure TTerrTiles.WriteAttrs(fs: TFileStream);
begin
  fs.write (FAttrs[0], sizeof(TTerrAttr) * FTileNum);
end;

function TTerrTiles.GetAttrs(index: Integer): TTerrAttr;
begin
  Result := FAttrs[index];
end;

procedure TTerrTiles.SetAttrs(index: Integer; const Value: TTerrAttr);
begin
  FAttrs[index] := Value;
end;

// implementation of TItemTiles class

procedure TItemTiles.SetTileNum(Value: Integer);
begin
  inherited SetTileNum(Value);
  SetLength(FAttrs, FTileNum);
end;

procedure TItemTiles.LoadAttrs(fs: TFileStream);
begin
  SetLength(FAttrs, FTileNum);
  fs.read (FAttrs[0], sizeof(TItemAttr) * FTileNum);
end;

procedure TItemTiles.WriteAttrs(fs: TFileStream);
begin
  fs.write (FAttrs[0], sizeof(TItemAttr) * FTileNum);
end;

function TItemTiles.GetAttrs(index: Integer): TItemAttr;
begin
  Result := FAttrs[index];
end;

procedure TItemTiles.SetAttrs(index: Integer; const Value: TItemAttr);
begin
  FAttrs[index] := Value;
end;

end.

