unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Grids, Menus, Math, Sprite, ComCtrls;

type
  TMainForm = class(TForm)
    pnlView: TPanel;
    pbxView: TPaintBox;
    mnuMain: TMainMenu;
    F1: TMenuItem;
    mnuChooseLevel: TMenuItem;
    N1: TMenuItem;
    mnuExit: TMenuItem;
    H1: TMenuItem;
    mnuAbout: TMenuItem;
    E1: TMenuItem;
    mnuEditTerr: TMenuItem;
    mnuEditItem: TMenuItem;
    mnuEditRole: TMenuItem;
    N2: TMenuItem;
    mnuShowGrid: TMenuItem;
    mnuShowSpeicalArea: TMenuItem;
    mnuResetMap: TMenuItem;
    V1: TMenuItem;
    stbMain: TStatusBar;
    N4: TMenuItem;
    mnuRemoveItem: TMenuItem;
    mnuSave: TMenuItem;
    N3: TMenuItem;
    mnuRestoreLevel: TMenuItem;
    mnuEditTerrItem: TMenuItem;
    mnuEditHiTerrItem: TMenuItem;
    Splitter1: TSplitter;
    pnlRight: TPanel;
    Splitter2: TSplitter;
    grdTile: TDrawGrid;
    tvwTiles: TTreeView;
    mnuResetAllMap: TMenuItem;
    mnuDrawEditLayerOnly: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure pbxViewPaint(Sender: TObject);
    procedure mnuEditRoleClick(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure pbxViewMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure mnuShowSomething(Sender: TObject);
    procedure mnuResetMapClick(Sender: TObject);
    procedure mnuRemoveItemClick(Sender: TObject);
    procedure mnuChooseLevelClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure mnuSaveClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure pbxViewMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pbxViewMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure mnuRestoreLevelClick(Sender: TObject);
    procedure grdTileDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure tvwTilesChange(Sender: TObject; Node: TTreeNode);
  private
    FEditLayer: Integer; // ثesҦ
    FLevelNo: Integer; // ثes誺d
    FModified: Boolean; // JdO_ʹL
    
    FTank: TMyTank; // ⪫
    FBackBitmap: TBitmap; // double-buffering ΪI bitmap
    
    FCursorX, FCursorY: Integer; // ƹЦm
    FButtonPressed: TMouseButton; // ƹ䪬A
    
    function GetCursorW: Integer;
    function GetCursorH: Integer;
    
    procedure DrawBackBitmap; // øsI bitmap
    procedure UpdateView; // sse
    procedure UpdateControlStatus;

    procedure SetLevelNo(Value: Integer);
    
    procedure MySaveFile; // xsaɮ
    function AskSaveMap: Boolean; // T{O_xsa
    function ValidateMap: Boolean; // ˬdaϬO_Xk
    
    property LevelNo: Integer read FLevelNo write SetLevelNo;
  public
    { Public declarations }
  end;
  
var
  MainForm: TMainForm;
  
implementation

uses util, tileunit, Mapunit, about, xFiles, xUtils;

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FEditLayer := 0; // w]aνsҦ
  FButtonPressed := mbMiddle; // ܥثeƹS

  FTank := TMyTank.Create; // D

  Tiles := TTiles.Create; // ͹Ϯw
  Map := tmap.Create; // da
  try
    FTank.LoadBits; // ŪJDϧ
    
    Tiles.LoadFromFile(AppDir + FN_TILE_ARCHIVE); // ŪJϮw
  except
    on E: Exception do
      begin
        ShowMessage(E.message);
        PostMessage(Handle, WM_CLOSE, 0, 0); // H asynchronous 覡
      end;
  end;

  // ŪJϮwyz
  ReadComponentResFile(ChangeFileExt(AppDir + FN_TILE_ARCHIVE, '.TVW'), tvwTiles);
  tvwTiles.Selected := tvwTiles.Items.GetFirstNode;

  // According tile num and size, adjust dimension of pnlView
  pnlView.ClientWidth := WORLD_WIDTH;
  pnlView.ClientHeight := WORLD_HEIGHT;

  ClientHeight := pnlView.Height + stbMain.Height;

  FBackBitmap := TBitmap.Create; // wĥ bitmap
  FBackBitmap.Width := TILE_WIDTH * TILE_NUM_X;
  FBackBitmap.Height := TILE_HEIGHT * TILE_NUM_Y;

  LevelNo := 1;
  UpdateControlStatus;

  // ƹЦm
  FCursorX := -1;
  FCursorY := -1;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FBackBitmap.Free;
  FTank.Free;
  Tiles.Free;
  Map.Free;
end;

procedure TMainForm.UpdateView;
begin
  DrawBackBitmap;
  pbxView.Canvas.Draw(0, 0, FBackBitmap);
end;

procedure TMainForm.pbxViewPaint(Sender: TObject);
begin
  UpdateView;
end;

procedure TMainForm.DrawBackBitmap;
var
  X, Y: Integer;
begin
  if (FEditLayer = LAYER_TERR) or not mnuDrawEditLayerOnly.Checked then Map.DrawMap(LAYER_TERR, FBackBitmap.Canvas) // øsaμh
  else begin // YSeaμh, N
    FBackBitmap.Canvas.Brush.Color := clBlack;
    FBackBitmap.Canvas.FillRect(FBackBitmap.Canvas.ClipRect);
  end;
  if (FEditLayer = LAYER_TERRITEM) or not mnuDrawEditLayerOnly.Checked then Map.DrawMap(LAYER_TERRITEM, FBackBitmap.Canvas); // øsaΪh
  if (FEditLayer = LAYER_ITEM) or not mnuDrawEditLayerOnly.Checked then Map.DrawMap(LAYER_ITEM, FBackBitmap.Canvas); // øs~h

  // pGϥΪ̷Qݮu, NNueX
  if mnuShowGrid.Checked then
    with FBackBitmap.Canvas do
    begin
      Pen.Color := clBlack;
      Pen.Style := psSolid;
      Pen.Width := 1;

      // eu
      for Y := 0 to TILE_NUM_Y - 1 do
      begin
        MoveTo(0, Y * TILE_HEIGHT);
        LineTo(TILE_NUM_X * TILE_WIDTH, Y * TILE_HEIGHT);
      end;

      // Aeu
      for X := 0 to TILE_NUM_X - 1 do
      begin
        MoveTo(X * TILE_WIDTH, 0);
        LineTo(X * TILE_WIDTH, TILE_NUM_Y * TILE_HEIGHT);
      end;
    end;

  // ϥΪ̷Qݤi樫ϰ
  if mnuShowSpeicalArea.Checked then
    with FBackBitmap.Canvas do
    begin
      Pen.Color := clRed;
      Pen.Width := 1;
      Pen.Style := psDash;
      Brush.Style := bsClear;

      for Y := 0 to TILE_NUM_Y - 1 do // v@ˬd
        for X := 0 to TILE_NUM_X - 1 do
          if not Map[LAYER_TERRITEM, X, Y].CanPass then
          begin // ~إ[WkWe쥪Uu
            Rectangle(X * TILE_WIDTH, Y * TILE_HEIGHT, (X + 1) * TILE_WIDTH - 1, (Y + 1) * TILE_HEIGHT - 1);
            MoveTo((X + 1) * TILE_WIDTH, Y * TILE_HEIGHT);
            LineTo(X * TILE_WIDTH, (Y + 1) * TILE_HEIGHT);
          end;
    end;

  FTank.Draw(FBackBitmap.Canvas); // eXϮ
  if (FEditLayer = LAYER_HITERRITEM) or not mnuDrawEditLayerOnly.Checked then Map.DrawMap(LAYER_HITERRITEM, FBackBitmap.Canvas); // øsaΪh
end;

procedure TMainForm.UpdateControlStatus;
const
  LayerDesc: array[0..4] of string = ('a', 'aΪ', '~', 'aΪ', 'D');
begin
  Caption := Format('%s -  %d (s説A: %s)',[Application.Title, FLevelNo, LayerDesc[FEditLayer]]);

  grdTile.Invalidate;
end;

procedure TMainForm.SetLevelNo(Value: Integer);
begin
  try
    // ŪaɤΨm
    Map.LevelNo := Value;
  except
    // YŪaɥ, Mia
    Map.ResetAllMap;

    // \Wj
    Map[LAYER_TERRITEM, 5, 11].TileNo := 2;
    Map[LAYER_TERRITEM, 6, 11].TileNo := 2;
    Map[LAYER_TERRITEM, 7, 11].TileNo := 2;
    Map[LAYER_TERRITEM, 5, 12].TileNo := 2;
    Map[LAYER_TERRITEM, 7, 12].TileNo := 2;

    // \WXl
    Map[LAYER_TERRITEM, 6, 12].TileNo := 5;
  end;
  FTank.X := Map.Role_X; // Nm Map 󤤧ۥX
  FTank.Y := Map.Role_Y;

  FLevelNo := Value;
  FModified := False; // aϩ|ʡ]M:p^
  UpdateControlStatus;
  UpdateView; // OѤFsCe
end;

procedure TMainForm.mnuEditRoleClick(Sender: TObject);
var
  bEnable: Boolean;
begin
  (Sender as TMenuItem).Checked := True;
  FEditLayer := (Sender as TMenuItem).Tag; // ]wsҦ

  UpdateControlStatus;
  UpdateView;
end;

procedure TMainForm.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.pbxViewMouseMove(Sender: TObject; Shift: TShiftState;
    X, Y: Integer);
begin
  // wg]Xd~F...
  if (X < 0) or (X >= TILE_WIDTH * TILE_NUM_X) or (Y < 0) or
    (Y >= TILE_HEIGHT * TILE_NUM_Y) then Exit;

  FCursorX := X div TILE_WIDTH; // y, HϮ欰
  FCursorY := Y div TILE_HEIGHT;

  pbxViewMouseDown(Sender, FButtonPressed, Shift, X, Y); // iHԦm񪫫~ĪG

  UpdateView; // øCe

  // eXثeҿϰ~
  with pbxView.Canvas do
  begin
    Pen.Width := 2;
    Pen.Color := clBlue;
    Brush.Style := bsClear;
    Rectangle(FCursorX * TILE_WIDTH, FCursorY * TILE_HEIGHT,
      (FCursorX + GetCursorW) * TILE_WIDTH, (FCursorY + GetCursorH) * TILE_HEIGHT);
  end;

  stbMain.SimpleText := Format('m: (%d, %d) jp: (%d, %d)',[FCursorX, FCursorY, GetCursorW, GetCursorH]);
end;

procedure TMainForm.mnuShowSomething(Sender: TObject);
begin
  (Sender as TMenuItem).Checked := not (Sender as TMenuItem).Checked;
  UpdateView;
end;

procedure TMainForm.mnuResetMapClick(Sender: TObject);
begin
  case (Sender as TComponent).Tag of
    0: if FEditLayer <= LAYER_MAX then Map.ResetMap(FEditLayer);
    1: Map.ResetAllMap;
  end;
  UpdateView;
end;

procedure TMainForm.mnuRemoveItemClick(Sender: TObject);
begin
  if (FCursorX = -1) or (FCursorY = -1) then Exit;

  if FEditLayer <= LAYER_MAX then
    Map[FEditLayer, FCursorX, FCursorY].TileNo := 0;
  UpdateView;
end;

procedure TMainForm.mnuChooseLevelClick(Sender: TObject);
var
  S: string;
begin
  if not AskSaveMap then Exit;

  S := IntToStr(FLevelNo);
  if InputQuery(Application.Title, 'Input Level No:', S) then
    LevelNo := StrToInt(S);
end;

function TMainForm.AskSaveMap: Boolean;
begin
  Result := True;
  if not FModified then Exit;

  // T{O_xsثes誺aɮ
  case YesNoCancelBox(Format('Do you want to save this map (level %d) ?',[FLevelNo])) of
    mrYes: MySaveFile;

    mrNo:;

    mrCancel: Result := False;
  end;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if not AskSaveMap then CanClose := False;
end;

procedure TMainForm.MySaveFile;
begin
  if not FModified then Exit;
  if not ValidateMap then Exit;

  Map.Role_X := FTank.X;
  Map.Role_Y := FTank.Y;
  Map.SaveToFile;
  FModified := False;
end;

procedure TMainForm.mnuSaveClick(Sender: TObject);
begin
  MySaveFile;
end;

// ˬdaϬO_Xk
function TMainForm.ValidateMap: Boolean;
var
  X, Y     : Integer;
  FlagCount: Integer;
begin
  Result := False;

  for Y := 0 to TILE_NUM_Y - 1 do
    for X := 0 to TILE_NUM_X - 1 do
    begin
      // ݬݦXXl
      if taFlag in Tiles[Map[LAYER_TERRITEM, X, Y].TileNo].Attr then Inc(FlagCount);
    end;

  if FlagCount = 0 then
  begin
    ShowMessage('SxX, ');
    Exit;
  end;

  Result := True;
end;

procedure TMainForm.mnuAboutClick(Sender: TObject);
begin
  with TAboutBox.Create(self) do
    try
      ShowModal;
    finally
      Free;
    end;
end;

procedure TMainForm.pbxViewMouseDown(Sender: TObject; Button: TMouseButton;
    Shift: TShiftState; X, Y: Integer);
var
  No    : Integer;
  MX, MY: Integer;
begin
  // OUƹ, tX OnMouseMove ƥBzk
  // ͩԦ]wĪG
  FButtonPressed := Button;

  if Button = mbMiddle then Exit; // ƹ䤣

  if Button = mbLeft then // O]w
  begin
    if tvwTiles.Selected = nil then Exit; // Sws

    No := Integer(tvwTiles.Selected.Data); // ϤsժYYs

    // W, WΥkWTBOĤZJXI, F
    if (FEditLayer <> LAYER_TERR) and (FCursorY = 0) and
      ((FCursorX = 0) or (FCursorX = TILE_NUM_X div 2 + 1) or
      (FCursorX = TILE_NUM_X - 1)) then Exit;

    if FEditLayer <= LAYER_MAX then
    begin
      for MY := 0 to GetCursorH - 1 do // Nsa\W
        for MX := 0 to GetCursorW - 1 do
          Map[FEditLayer, FCursorX + MX, FCursorY + MY].TileNo :=
          No + Tiles[No].xNum * (MY + grdTile.Selection.Top) +
          (MX + grdTile.Selection.Left);
    end else
    begin
      // ⤣iH\biʪaΤW
      if not Map[LAYER_TERRITEM, FCursorX, FCursorY].CanPass then
        Exit;

      FTank.X := FCursorX * TILE_WIDTH; // ]wDlm
      FTank.Y := FCursorY * TILE_HEIGHT;
    end;
  end else // kOM
  begin
    if FEditLayer > LAYER_MAX then Exit; // ⤣βM

    for MY := 0 to GetCursorH - 1 do
      for MX := 0 to GetCursorW - 1 do // Ma
        Map[FEditLayer, FCursorX + MX, FCursorY + MY].TileNo := 0;
  end;

  FModified := True; // aϤw
  UpdateView; // saϵe
end;

procedure TMainForm.pbxViewMouseUp(Sender: TObject; Button: TMouseButton;
    Shift: TShiftState; X, Y: Integer);
begin
  FButtonPressed := mbMiddle; // ]w mbMiddle ܨϥΪ̤w}ƹ
end;

procedure TMainForm.mnuRestoreLevelClick(Sender: TObject);
begin
  LevelNo := LevelNo;
end;

procedure TMainForm.grdTileDrawCell(Sender: TObject; ACol, ARow: Integer;
    Rect: TRect; State: TGridDrawState);
var
  No  : Integer;
  X, Y: Integer;
begin
  if FEditLayer > LAYER_MAX then Exit;
  if tvwTiles.Selected = nil then Exit;

  No := Integer(tvwTiles.Selected.Data);
  if (ARow * grdTile.ColCount + ACol >= Tiles[No].xNum * Tiles[No].yNum) then Exit;

  Inc(No, ARow * grdTile.ColCount + ACol);
  if No >= Tiles.TileNum then Exit;

  with grdTile.Canvas do // NϧεeX
    Draw(Rect.Left, Rect.Top, Tiles[No].Bitmap);
end;

procedure TMainForm.tvwTilesChange(Sender: TObject; Node: TTreeNode);
var
  No: Integer;
begin
  if tvwTiles.Selected = nil then Exit;

  No := Integer(tvwTiles.Selected.Data); // oثeϮsսs

  with grdTile, grdTile.Selection do
  begin
    RowCount := Tiles[No].yNum; // ]w grid jp, ܾӹϮs
    ColCount := Tiles[No].xNum;

    Left := 0;
    Top := 0;
    Right := Tiles[No].xNum - 1;
    Bottom := Tiles[No].yNum - 1;
  end;

  grdTile.Invalidate;
end;

// oܰϰ쪺eΰ
function TMainForm.GetCursorW: Integer;
begin
  with grdTile.Selection do
    Result := Right - Left + 1;
end;

function TMainForm.GetCursorH: Integer;
begin
  with grdTile.Selection do
    Result := Bottom - Top + 1;
end;

end.

